3dtiles-tool
========

[![NPM Package][npm]][npm-url]

#### 3DTiles Generate Tool ####

### Usage ###

3DTiles规范包括了诸如GLTF、B3DM、PNTS、JSON等多种类型的文件，本工具库主要是为生成及组装这些文件提供便利。当前，仅支持从Three.js对象导出GLB及生成B3DM文件两大功能，后续还将不断完善。

首先，根据Three.js对象生成GLB文件，其中batchId数组需要依据业务需求填充相应值，具体请参考3DTiles规范文档。
```javascript
import { BatchTableMap, Batched3DModel, exportGlbBufferFromTHREE } from "tg-3dtiles-tool"
import * as THREE from "three"

const blockFaceAttribute = {
    batchId: new Uint16Array(visibleTriangleFaceCount * 3),
    position: new Float64Array(visibleTriangleFaceCount * 9),
    normal: new Float64Array(visibleTriangleFaceCount * 9),
    color: new Float32Array(visibleTriangleFaceCount * 12),
}

const blockFaceGeometry = new THREE.BufferGeometry()
blockFaceGeometry.setAttribute('position', new THREE.Float32BufferAttribute(blockFaceAttribute.position, 3))
blockFaceGeometry.setAttribute('color', new THREE.Float32BufferAttribute(blockFaceAttribute.color, 4))
blockFaceGeometry.setAttribute('normal', new THREE.Float32BufferAttribute(blockFaceAttribute.normal, 3))
blockFaceGeometry.setAttribute('batchId', new THREE.Uint16BufferAttribute(blockFaceAttribute.batchId, 1))

const blockFaceMesh = new THREE.Mesh(blockFaceGeometry, new THREE.MeshPhongMaterial({
vertexColors: THREE.FaceColors,
flatShading: true,
}))

const glbBuffer = await exportGlbBufferFromTHREE(blockFaceMesh)
```

接着，根据业务需求制作batchTable和featureTable对象。
```javascript
import { BatchTableMap, Batched3DModel } from "tg-3dtiles-tool"

const batchLength = 100
const batchTableMap = new BatchTableMap()

batchTableMap.set("verticeIndex", new Uint16Array(batchLength))
batchTableMap.set("cellIndex", new Uint16Array(3 * batchLength))
```

最后，制作Batched3DModel对象，并调用export()方法即可将以上三者拼接成B3DM文件，该方法返回值为ArrayBuffer对象，可直接作为二进制数据流存入数据库或写入文件。
```javascript
const b3dm = new Batched3DModel()
b3dm.glbBuffer = glbBuffer
b3dm.batchTableMap = batchTableMap

const buffer = b3dm.export()
```

[npm-url]: https://www.npmjs.com/package/3dtiles-tool